
from rest_framework import serializers


from django_resaas.models.documento import Documento


class DocumentoSerializer(serializers.ModelSerializer):
    class Meta:
        model = Documento
        fields = "__all__"



import os
from django.db import models
from django_resaas.core.utils.full_path import FullPath


class FileFieldsMixin:

    permanent_fields_files = []

    def _file_representation(self, request, file, field_name):
        if not file:
            return None

        try:
            url = FullPath.url(
                request,
                file.url,
                temporary=field_name not in self.permanent_fields_files
            )

            name = os.path.basename(file.name)
            ext = os.path.splitext(name)[1].lstrip('.').lower()

            return {
                "url": url,
                "name": name,
                "ext": ext,
                "size": getattr(file, "size", None),
            }
        except Exception:
            return None



from rest_framework import viewsets
from django_resaas.models.documento import Documento
from django_resaas.data.documento.serializers.documento import DocumentoSerializer


class  DocumentoAPIView(viewsets.ModelViewSet):
    serializer_class = DocumentoSerializer
    queryset = Documento.objects.all()
    def get_queryset(self):
        return self.queryset.filter().order_by('-id')

   


"""
URL configuration for dev project.

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/5.2/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""

from django.urls import include
from django.urls import path

from rest_framework import routers

from rest_framework_simplejwt.views import TokenRefreshView

from django_resaas.data.bootstrap.views.bootstrap import TenantAPIView


# ─────────────────────────────
# User / Auth views
# ─────────────────────────────
from django_resaas.data.user.views.login import LoginAPIView
from django_resaas.data.user.views.logins import LoginsAPIView
from django_resaas.data.user.views.logout import LogoutAPIView
from django_resaas.data.user.views.me import MeAPIView
from django_resaas.data.user.views.verify_email import VerifyEmail
from django_resaas.data.user.views.change_password_email import ChangePasswordEmailAPIView
from django_resaas.data.user.views.change_password_mobile import ChangePasswordMobileAPIView
from django_resaas.data.user.views.request_password_reset_email import RequestPasswordResetEmailAPIView
from django_resaas.data.user.views.password_token_check import PasswordTokenCheckAPIView
from django_resaas.data.user.views.set_new_password import SetNewPasswordAPIView
from django_resaas.data.user.views.mail import MailAPIView

# ─────────────────────────────
# Data / API views
# ─────────────────────────────
from django_resaas.data.entidade.views.entidade import EntidadeAPIView
from django_resaas.data.entidade.views.site import SiteAPIView
from django_resaas.data.tipo_entidade.views.tipo_entidade import TipoEntidadeAPIView
from django_resaas.data.group.views.grupo import GrupoAPIView
from django_resaas.data.sucursal.views.sucursal import SucursalAPIView
from django_resaas.data.sucursal_user.views.sucursal_user import SucursalUserAPIView
from django_resaas.data.sucursal_user_group.views.sucursal_user_group import SucursalUserGroupAPIView
from django_resaas.data.documento.views.documento import DocumentoAPIView
from django_resaas.data.tipo_documento.views.tipo_documento import TipoDocumentoAPIView



from django_resaas.data.traducao.views.traducao import TraducaoAPIView
from django_resaas.data.idioma.views.idioma import IdiomaAPIView
from django_resaas.data.ficheiro.views.ficheiro import FicheiroAPIView
from django_resaas.data.permission.views.permission import PermissionAPIView
from django_resaas.data.modelo.views.modelo import ModeloAPIView
from django_resaas.data.user.views.user import UserAPIView
from django_resaas.data.pessoa.views.pessoa import PessoaAPIView
from django_resaas.data.theme.views.theme import ThemeAPIView
from django_resaas.data.layout_setting.views.layout_setting import LayoutSettingAPIView
from django_resaas.management.apicommands.view.scaffold import ScaffoldAPIView
from django_resaas.management.apicommands.view.modulo_schema import ModuloSchemaAPIView, RelationsAPIView

from django_resaas.data.pdf.views.invoice import invoice_pdf

from django_resaas.view import home
from django_resaas.view import deploy_github, deploy_status, deploy_releases, deploy_logs, deploy_rollback
from django_resaas.core.utils.autoload_urls import build_saas_urls




# ─────────────────────────────
# Router
# ─────────────────────────────
routerdjango_resaas = routers.DefaultRouter()
routerauth = routers.DefaultRouter()

routerdjango_resaas.register("ficheiros", FicheiroAPIView, basename="ficheiros")
routerdjango_resaas.register("idiomas", IdiomaAPIView, basename="idiomas")
routerdjango_resaas.register("traducaos", TraducaoAPIView, basename="traducaos")
routerdjango_resaas.register("themes", ThemeAPIView, basename="themes")
routerdjango_resaas.register("layoutsettings", LayoutSettingAPIView, basename="layoutsettings")
routerdjango_resaas.register("tipodocumentos", TipoDocumentoAPIView, basename="tipodocumentos")
routerdjango_resaas.register("documentos", DocumentoAPIView, basename="documentos")


routerdjango_resaas.register("sucursalusergroups", SucursalUserGroupAPIView, basename="sucursalusergroups")
routerdjango_resaas.register("sucursalusers", SucursalUserAPIView, basename="sucursalusers")

routerdjango_resaas.register("tipoentidades", TipoEntidadeAPIView, basename="tipo_entidades")
routerdjango_resaas.register("entidades", EntidadeAPIView, basename="entidades")
routerdjango_resaas.register("sucursals", SucursalAPIView, basename="sucursais")
routerdjango_resaas.register("users", UserAPIView, basename="users")
routerdjango_resaas.register("pessoas", PessoaAPIView, basename="pessoas")

routerauth.register("groups", GrupoAPIView, basename="groups")
routerauth.register("permissions", PermissionAPIView, basename="permissions")
routerdjango_resaas.register("modelos", ModeloAPIView, basename="modelos")
routerdjango_resaas.register("modulos", ModuloSchemaAPIView, basename="modulos")
routerdjango_resaas.register("scaffolds", ScaffoldAPIView, basename="scaffolds")



urlpatterns = [

    path('', home, name='home'),

    path("deploy/github/", deploy_github),
    path("deploy/status/", deploy_status),
    path("deploy/releases/", deploy_releases),
    path("deploy/logs/", deploy_logs),
    path("deploy/rollback/", deploy_rollback),
    

    path("django_resaas/", include(routerdjango_resaas.urls)),
    path("auth/", include(routerauth.urls)),
    path("django_resaas/relations/", RelationsAPIView.as_view()),


    path("site/", SiteAPIView.as_view(), name="site"),

    path("setup/", TenantAPIView.as_view(), name="setup"),
    path("login/", LoginAPIView.as_view(), name="login"),
    path("logout/", LogoutAPIView.as_view(), name="logout"),
    path("me/", MeAPIView.as_view(), name="me"),

    path("email/verify/", VerifyEmail.as_view(), name="email_verify"),
    path("refresh_token/", TokenRefreshView.as_view(), name="token_refresh"),

    path("logins/", LoginsAPIView.as_view(), name="logins"),

    path("password/change/email/", ChangePasswordEmailAPIView.as_view(), name="change_password_email"),
    path("password/change/mobile/", ChangePasswordMobileAPIView.as_view(), name="change_password_mobile"),
    path("password/reset/email/", RequestPasswordResetEmailAPIView.as_view(), name="request_password_reset_email"),
    path("password/reset/<uidb64>/<token>/", PasswordTokenCheckAPIView.as_view(), name="password_reset_confirm"),
    path("password/reset/complete/", SetNewPasswordAPIView.as_view(), name="password_reset_complete"),

    path("mail/", MailAPIView.as_view(), name="mail"),
    path("pdf/invoice/<int:invoice_id>/", invoice_pdf, name="invoice_pdf"),

]







from django.core.management.base import BaseCommand
from django.contrib.auth import get_user_model
from getpass import getpass
from django_resaas.models.tipo_entidade import TipoEntidade
from django_resaas.models.entidade import Entidade
from django_resaas.models.sucursal import Sucursal
from django_resaas.models.entidade_user import EntidadeUser
from django_resaas.models.sucursal_user import SucursalUser
from django_resaas.models.sucursal_user_group import SucursalUserGroup
from django.contrib.auth.models import Group
from django_resaas.core.services.frontend_service import FrontEndService
from django_resaas.core.services.idioma_service import IdiomaService
from django_resaas.models.modulo import Modulo
from django_resaas.models.tipo_entidade_modulo import TipoEntidadeModulo
from django_resaas.models.entidade_modulo import EntidadeModulo

from django_resaas.models.tipo_entidade_group import TipoEntidadeGroup
from django_resaas.models.entidade_group import EntidadeGroup
from django_resaas.models.sucursal_group import SucursalGroup

User = get_user_model()


class Command(BaseCommand):
    help = "Cria um superuser estático se não existir"

    def handle(self, *args, **options):
        email = "root@co.mz"
        username = "root"

        FrontEndService.load_defaults( stdout=self.stdout,  style=self.style  )
        IdiomaService.load_defaults( stdout=self.stdout, style=self.style )

        user = User.objects.filter(email=email).first()
        exist = False
        if user :
            exist = True
            self.stdout.write(
                self.style.WARNING("\nSuperuser já existe")
            )
            self.stdout.write(
                self.style.HTTP_INFO("  Username: \t") + self.style.SUCCESS(user.username)
            )
            self.stdout.write(
                self.style.HTTP_INFO("  Email: \t") + self.style.SUCCESS(user.email) 
            )

            self.stdout.write(
                self.style.WARNING("\n")
            )




        # 🔐 pedir password pelo teclado (sem mostrar)
        while True:
            password = getpass("Password do superuser: ")
            password_confirm = getpass("Confirme a password: ")

            if not password:
                self.stdout.write(
                    self.style.ERROR("A password não pode estar vazia")
                )
                continue

            if password != password_confirm:
                self.stdout.write(
                    self.style.ERROR("As passwords não coincidem")
                )
                continue

            break
        if not exist:
            user = User.objects.create_superuser(
                email=email,
                username=username,
                password=password,
            )

            user.set_password(password)
            user.is_verified_email= True
            user.save()


        data = {
            "tipo_entidade": "SaaS",
            "entidade": "Mytech",
            "sucursal": "Sede",
            "grupo": ["Guest", "root", "Admin"],
        }

        # ------------------------
        # 1. TipoEntidade
        # ------------------------
        tipo_entidade, _ = TipoEntidade.objects.get_or_create(
            nome=data["tipo_entidade"],
            defaults={"estado": 1}
        )

        # ------------------------
        # 2. Entidade
        # ------------------------
        entidade, created_entidade = Entidade.objects.get_or_create(
            nome=data["entidade"],
            tipo_entidade=tipo_entidade
        )

        # ManyToMany → DEPOIS
        entidade.admins.add(user)

        EntidadeUser.objects.get_or_create(
            user=user,
            entidade=entidade
        )

        # ------------------------
        # 3. Sucursal
        # ------------------------
        sucursal, _ = Sucursal.objects.get_or_create(
            nome=data["sucursal"],
            entidade=entidade
        )

        SucursalUser.objects.get_or_create(
            user=user,
            sucursal=sucursal
        )

        # ------------------------
        # 4. Grupo
        # ------------------------
        for g in data["grupo"]:

            grupo, _ = Group.objects.get_or_create(
                name=g
            )

            SucursalUserGroup.objects.get_or_create(
                user=user,
                sucursal=sucursal,
                group=grupo
            )

            user.groups.add(grupo)

            TipoEntidadeGroup.objects.get_or_create(
                tipo_entidade=tipo_entidade,
                group=grupo
            )

            EntidadeGroup.objects.get_or_create(
                entidade=entidade,
                group=grupo
            )

            SucursalGroup.objects.get_or_create(
                sucursal=sucursal,
                group=grupo
            )

        self.stdout.write(self.style.WARNING(f"\n"))

        for name in ['django_resaas',]:
            modulo, _ = Modulo.objects.get_or_create(
                nome=name,
                defaults={"estado": 1}
            )

            tipo_entidade_modulo, _ = TipoEntidadeModulo.objects.get_or_create(
                modulo=modulo,
                tipo_entidade=tipo_entidade,
                defaults={"estado": 1}
            )

            entidade_modulo, _ = EntidadeModulo.objects.get_or_create(
                modulo=modulo,
                entidade=entidade,
                defaults={"estado": 1}
            )

            self.stdout.write(self.style.WARNING(f"✔ {'Modulo:':20} {modulo.nome}"))

        self.stdout.write(self.style.HTTP_INFO(f""))
        self.stdout.write(self.style.HTTP_INFO(f""))
        self.stdout.write(self.style.HTTP_INFO(f"{'':10} {'✔ Superuser criado:'}"))
        self.stdout.write(self.style.HTTP_INFO(f""))
        self.stdout.write(self.style.HTTP_INFO(f"{'Email:':20} {user.email}"))
        self.stdout.write(self.style.SUCCESS(f"{'User:':20} {user.username}"))
        self.stdout.write(self.style.HTTP_SUCCESS(f"{'TipoEntidade:':20} {tipo_entidade.nome}"))
        self.stdout.write(self.style.HTTP_NOT_MODIFIED(f"{'Entidade:':20} {entidade.nome}"))
        self.stdout.write(self.style.HTTP_SERVER_ERROR(f"{'Sucursal:':20} {sucursal.nome}"))
        self.stdout.write(self.style.WARNING(f"{'Grupos:':20} {data['grupo']}"))
        self.stdout.write(self.style.ERROR("⚠️  Guarde estas credenciais com segurança"))
        self.stdout.write(self.style.HTTP_INFO(f""))
        self.stdout.write(self.style.HTTP_INFO(f""))



       


from django.conf import settings
from django.db import models
import uuid
# good
from django.utils import timezone
import os 
from .mixins.model.label_value import LabelValueMixin


def file_path(instance, file_name, pasta=""):
    ext = os.path.splitext(file_name)[1].lower()
    unique_name = f"{uuid.uuid4()}{ext}"

    pasta = pasta.strip("/")

    return (
        f"{instance.entidade.tipo_entidade.id}/"
        f"{instance.entidade.id}/"
        f"{instance.id}/"
        f"{pasta}/{unique_name}" if pasta else
        f"{instance.entidade.tipo_entidade.id}/"
        f"{instance.entidade.id}/"
        f"{instance.id}/{unique_name}"
    )
    

class SoftDeleteQuerySet(models.QuerySet):
    def alive(self):
        return self.filter(deleted_at__isnull=True)

    def deleted(self):
        return self.filter(deleted_at__isnull=False)

    def soft_delete(self):
        return self.update( deleted_at=timezone.now())

    def restore(self):
        return self.update( deleted_at=None)

    def hard_delete(self):
        return super().delete()


class SoftDeleteManager(models.Manager):
    def get_queryset(self):
        return SoftDeleteQuerySet(self.model, using=self._db).alive()

class DeletedManager(models.Manager):
    def get_queryset(self):
        return SoftDeleteQuerySet(self.model, using=self._db).deleted()


class AllObjectsManager(models.Manager):
    def get_queryset(self):
        return SoftDeleteQuerySet(self.model, using=self._db)


class SoftBaseModel(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    deleted_at = models.DateTimeField(null=True, blank=True)

    objects = SoftDeleteManager()      # só ativos
    all_objects = AllObjectsManager()  # tudo (ativos + apagados)
    deleted_objects = DeletedManager()  #  só apagados

    class Meta:
        abstract = True


    def delete(self, using=None, keep_parents=False, user=None):
        self.deleted_at = timezone.now()
        if user:
            self.updated_by = user
        self.save(update_fields=["deleted_at", "updated_by"])

    def restore(self, user=None):
        self.deleted_at = None
        if user:
            self.updated_by = user
        self.save(update_fields=["deleted_at", "updated_by"])

    def hard_delete(self):
        super().delete()

class TimeModel(LabelValueMixin,SoftBaseModel):
    created_by = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="%(class)s_created",
        
    )

    updated_by = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="%(class)s_updated"
    )
    
    estado = models.IntegerField(
        default=0,
        null=True,
        choices=((0, 'Inativo'), (1, 'Activo')),
    )
    class Meta:
        abstract = True

class BaseModel(TimeModel):
    entidade = models.ForeignKey(
        "Entidade",
        on_delete=models.CASCADE,
        related_name="%(class)s_entidade"
    )

    sucursal = models.ForeignKey(
        "Sucursal",
        on_delete=models.CASCADE,
        related_name="%(class)s_sucursal"
    )


    class Meta:
        abstract = True







from rest_framework import serializers


from django_resaas.models.pessoa import Pessoa


class PessoaSerializer(serializers.ModelSerializer):
    class Meta:
        model = Pessoa
        fields = "__all__"



from rest_framework import serializers

from django_resaas.models.user import User
from django_resaas.core.utils.translate import Translate


class RegisterSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = ('id', 'email', 'username', 'password', 'mobile')
        extra_kwargs = {
            'password': {'write_only': True}
        }

    def validate(self, attrs):
        request = self.context.get('request')
        username = attrs.get('username', '')

        if not username.isalnum():
            raise serializers.ValidationError(
                {
                    'username': Translate.tdc(
                        request,
                        'O nome de utilizador deve conter apenas caracteres alfanuméricos'
                    )
                }
            )

        return attrs

    def create(self, validated_data):
        return User.objects.create_user(**validated_data)



from rest_framework import serializers

from django.contrib.contenttypes.models import ContentType


class ModeloSerializer(serializers.ModelSerializer):
    class Meta:
        model = ContentType
        fields = [
            "id",
            "app_label",
            "model",
        ]



# =========================
# Django
# =========================
from django_resaas.core.base.admin import BaseAdmin, all_fields
from django.contrib import admin
from django.contrib.auth import get_user_model
from django.contrib.auth.models import Permission

from django.contrib.contenttypes.admin import GenericTabularInline
from .models import Documento


class DocumentoInline(GenericTabularInline):
    model = Documento
    extra = 1


# =========================
# ds – models
# =========================
from django_resaas.models.traducao import Traducao
from django_resaas.models.idioma import Idioma
from django_resaas.models.entidade import Entidade
from django_resaas.models.entidade_user import EntidadeUser
from django_resaas.models.entidade_modulo import EntidadeModulo
from django_resaas.models.entidade_group import EntidadeGroup

from django_resaas.models.sucursal import Sucursal
from django_resaas.models.sucursal_user import SucursalUser
from django_resaas.models.sucursal_user_group import SucursalUserGroup
from django_resaas.models.sucursal_group import SucursalGroup

from django_resaas.models.tipo_entidade import TipoEntidade
from django_resaas.models.tipo_entidade_modulo import TipoEntidadeModulo
from django_resaas.models.tipo_entidade_modelo import TipoEntidadeModelo
from django_resaas.models.entidade_modelo import EntidadeModelo
from django_resaas.models.ficheiro import Ficheiro
from django_resaas.models.user_login import UserLogin
from django_resaas.models.modulo import Modulo
from django_resaas.models.front_end import FrontEnd
from django_resaas.models.modelo_extra import ModeloExtra
from django_resaas.models.documento import TipoDocumento, Documento


# =========================
# User model
# =========================
User = get_user_model()

admin.site.site_title = 'Django Rest SaaS'
admin.site.index_title = 'Django Rest SaaS'



@admin.register(TipoDocumento)
class TipoDocumentoAdmin(BaseAdmin):
    list_display = ('nome', 'detalhes')
    
@admin.register(Documento)
class DocumentoAdmin(BaseAdmin):
    list_display = ('tipo', 'numero', 'data_emissao', 'data_validade')
    list_filter = ('tipo',)

@admin.register(Traducao)
class TraducaoAdmin(BaseAdmin):
    def get_list_display(self, request): return all_fields(self.model)
    list_display_links = ('id',)

from django_resaas.models.theme import Theme
@admin.register(Theme)
class ThemeAdmin(BaseAdmin):
    def get_list_display(self, request): return all_fields(self.model)
    list_display_links = ('id',)

from django_resaas.models.layout_setting import LayoutSetting
@admin.register(LayoutSetting)
class LayoutSettingAdmin(BaseAdmin):
    def get_list_display(self, request): return all_fields(self.model)
    list_display_links = ('id',)

from django_resaas.models.cors_allowed_origin import CorsAllowedOrigin
@admin.register(CorsAllowedOrigin)
class CorsAllowedOriginAdmin(BaseAdmin):
    def get_list_display(self, request): return all_fields(self.model)
    list_display_links = ('id',)


from django_resaas.models.theme import Typography
@admin.register(Typography)
class TypographyAdmin(BaseAdmin):
    def get_list_display(self, request): return all_fields(self.model)
    list_display_links = ('id',)

from django_resaas.models.layout_setting import AnimationSetting
@admin.register(AnimationSetting)
class AnimationSettingAdmin(BaseAdmin):
    def get_list_display(self, request): return all_fields(self.model)
    list_display_links = ('id',)


@admin.register(EntidadeGroup)
class EntidadeGroupAdmin(BaseAdmin):
    def get_list_display(self, request): return all_fields(self.model)
    list_display_links = ('id',)


@admin.register(Ficheiro)
class FicheiroAdmin(BaseAdmin):
    def get_list_display(self, request): return all_fields(self.model)
    list_display_links = ('id',)

@admin.register(TipoEntidadeModelo)
class TipoEntidadeModeloAdmin(BaseAdmin):
    def get_list_display(self, request): return all_fields(self.model)
    list_display_links = ('id',)

@admin.register(EntidadeModelo)
class EntidadeModeloAdmin(BaseAdmin):
    def get_list_display(self, request): return all_fields(self.model)
    list_display_links = ('id',)


@admin.register(Permission)
class PermissionAdmin(admin.ModelAdmin):
    def get_list_display(self, request): return all_fields(self.model)
    list_display_links = ('id',)
    search_fields = ['id', 'name']

from .models import Pessoa

@admin.register(Pessoa)
class PessoaAdmin(admin.ModelAdmin):
    def get_list_display(self, request): return all_fields(self.model)
    inlines = [DocumentoInline]


@admin.register(FrontEnd)
class FrontEndAdmin(BaseAdmin):
    def get_list_display(self, request): return all_fields(self.model)


@admin.register(Idioma)
class IdiomaAdmin(BaseAdmin):
    def get_list_display(self, request): return all_fields(self.model)
    list_display_links = ('id',)
    search_fields = ['id', 'nome']


@admin.register(TipoEntidade)
class TipoEntidadeAdmin(BaseAdmin):
    def get_list_display(self, request): return all_fields(self.model)
    list_display_links = ('id', 'nome')
    search_fields = ['nome']


@admin.register(User)
class UserAdmin( BaseAdmin):
    
    def get_list_display(self, request):
        exclude = ['password']
        fields = all_fields(self.model)
        return [f for f in fields if f not in exclude]

    list_display_links = ('id', 'username', 'email')
    search_fields = ['username', 'mobile', 'email']

    readonly_fields = ('password',)


@admin.register(UserLogin)
class UserLoginAdmin(BaseAdmin):
    def get_list_display(self, request): return all_fields(self.model)
    list_display_links = ('id',)
    search_fields = ['local_nome', 'dispositivo', 'user']


@admin.register(Entidade)
class EntidadeAdmin(BaseAdmin):
    def get_list_display(self, request): return all_fields(self.model)
    list_display_links = ('id', 'nome')
    search_fields = ['nome']

    def admin_list(self, obj):
        return ', '.join(u.username for u in obj.admins.all())
    admin_list.short_description = 'admins'


@admin.register(EntidadeUser)
class EntidadeUserAdmin(BaseAdmin):
    def get_list_display(self, request): return all_fields(self.model)
    list_display_links = ('id', 'user')
    search_fields = ['user']


@admin.register(Sucursal)
class SucursalAdmin(BaseAdmin):
    def get_list_display(self, request): return all_fields(self.model)
    list_display_links = ('id', 'nome')
    search_fields = ['nome']


@admin.register(SucursalGroup)
class SucursalGroupAdmin(BaseAdmin):
    def get_list_display(self, request): return all_fields(self.model)
    list_display_links = ('id', 'sucursal', 'group')
    search_fields = ['sucursal', 'group']


@admin.register(SucursalUser)
class SucursalUserAdmin(BaseAdmin):
    def get_list_display(self, request): return all_fields(self.model)
    list_display_links = ('id', 'sucursal', 'user')
    search_fields = ['sucursal', 'user']


@admin.register(SucursalUserGroup)
class SucursalUserGroupAdmin(BaseAdmin):
    def get_list_display(self, request): return all_fields(self.model)
    list_display_links = ('id', 'sucursal', 'user', 'group')
    search_fields = ['sucursal', 'user', 'group']


@admin.register(EntidadeModulo)
class EntidadeModuloAdmin(BaseAdmin):
    def get_list_display(self, request): return all_fields(self.model)
    list_display_links = ('id', 'entidade', 'modulo')
    search_fields = ['entidade', 'modulo']


@admin.register(TipoEntidadeModulo)
class TipoEntidadeModuloAdmin(BaseAdmin):
    def get_list_display(self, request): return all_fields(self.model)
    list_display_links = ('id', 'tipo_entidade', 'modulo')
    search_fields = ['tipo_entidade', 'modulo']

@admin.register(ModeloExtra)
class ModeloExtraAdmin(BaseAdmin):
    def get_list_display(self, request): return all_fields(self.model)
    list_display_links = ('id', 'modelo')
    search_fields = [ 'icon', 'modelo', 'url', 'datails', 'permission']

@admin.register(Modulo)
class ModuloAdmin(BaseAdmin):
    def get_list_display(self, request): return all_fields(self.model)








from rest_framework import serializers


from django_resaas.models.documento import TipoDocumento


class TipoDocumentoSerializer(serializers.ModelSerializer):
    class Meta:
        model = TipoDocumento
        fields = "__all__"



import uuid
from django.db import models
from django_resaas.core.base.models import TimeModel

from django.contrib.contenttypes.fields import GenericRelation


import uuid
from django.db import models
from django.contrib.contenttypes.fields import GenericRelation


class Pessoa(TimeModel):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)

    user = models.OneToOneField(
        'django_resaas.User',
        on_delete=models.CASCADE,
        related_name='pessoa',
        null=True,
        blank=True
    )

    # 📌 Dados básicos
    nome = models.CharField(max_length=100, null=True)
    apelido = models.CharField(max_length=100, null=True)
    nome_completo = models.CharField(max_length=200, null=True, blank=True)

    # 📌 Identificação
    GENERO_CHOICES = [
        ('M', 'Masculino'),
        ('F', 'Feminino'),
        ('O', 'Outro'),
    ]

    genero = models.CharField(
        max_length=1,
        choices=GENERO_CHOICES,
        null=True,
        blank=True
    )

    data_nascimento = models.DateField(null=True, blank=True)
    nacionalidade = models.CharField(max_length=100, null=True, blank=True)

    # 📌 Contactos
    email = models.EmailField(null=True, blank=True, unique=True)
    telefone = models.CharField(max_length=20, null=True, blank=True)
    telefone_alternativo = models.CharField(max_length=20, null=True, blank=True)

    # 📌 Endereço
    endereco = models.ForeignKey(
        'django_resaas.Endereco',
        on_delete=models.SET_NULL,  # 🔥 melhor que CASCADE
        null=True,
        blank=True,
        related_name='pessoas'
    )

    # 📌 Documentos
    documentos = GenericRelation('django_resaas.Documento')



    # 📌 Extra
    observacoes = models.TextField(null=True, blank=True)

    class Meta:
        verbose_name = 'Pessoa'
        verbose_name_plural = 'Pessoas'
        ordering = ['nome']
        indexes = [
            models.Index(fields=['nome']),
            models.Index(fields=['email']),
        ]

    def save(self, *args, **kwargs):
        # 🔥 Gera nome completo automaticamente
        if self.nome and self.apelido:
            self.nome_completo = f"{self.nome} {self.apelido}".strip()

        # 🔥 Normaliza email
        if self.email:
            self.email = self.email.lower()

        super().save(*args, **kwargs)

    def idade(self):
        from datetime import date
        if self.data_nascimento:
            today = date.today()
            return today.year - self.data_nascimento.year - (
                (today.month, today.day) < (self.data_nascimento.month, self.data_nascimento.day)
            )
        return None

    def __str__(self):
        return self.nome_completo or self.nome or "Sem nome"


import base64

import pyotp

from rest_framework import generics, status
from rest_framework.response import Response

from django_resaas.models.user import User
from django_resaas.core.utils.translate import Translate
from django_resaas.core.utils.generate_key_otp import generateKeyOTP


class ChangePasswordMobileAPIView(generics.GenericAPIView):
    def post(self, request):
        phone = request.data.get('mobile')
        user = User.objects.get(mobile=phone)

        keygen = generateKeyOTP()
        key = base64.b32encode(
            keygen.returnValue(phone).encode()
        )
        otp = pyotp.HOTP(key)

        if otp.verify(request.data.get("otp"), user.counter):
            user.set_password(request.data.get('password'))
            user.save()
            return Response(
                {
                    'alert_success': Translate.tdc(
                        request,
                        'Senha redefinida com sucesso'
                    )
                },
                status=status.HTTP_202_ACCEPTED
            )

        return Response(
            {
                'alert_error': Translate.tdc(
                    request,
                    'OTP inválido ou expirado'
                )
            },
            status=status.HTTP_400_BAD_REQUEST
        )
